/*------------------------------------------------------------------------- 
File name   : uart_rx_bfm.e
Title       : uart eVC cell structure
Project     : uart eVC
Developers  : Pinhas Krengel http://bknpk.no-ip.biz/my_web/index.html
Created     : Dec 2011
Description : This file drives data to uart RX.
Notes       : 
--------------------------------------------------------------------------- 
==== This source file may be used and distributed without ====
==== restriction provided that this copyright statement is not ====
==== removed from the file and that any derivative work contains ====
==== the original copyright notice and the associated disclaimer.====
==== ====
==== THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY ====
==== EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED ====
==== TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS ====
==== FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE AUTHOR ====
==== OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, ====
==== INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES ====
==== (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE ====
==== GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR ====
==== BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF ====
==== LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT ====
==== (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT ====
==== OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE ====
==== POSSIBILITY OF SUCH DAMAGE. ====
-------------------------------------------------------------------------*/ 
<'
package pk_rx;

unit uart_rx_bfm_u {
  rx_driver : uart_rx_driver_u;
  name      : uart_rx_env_name_t;
  sig       : sig_map_u;

  rx_driver_clk_tcm() @sig.clk_r_ev is {
    while(TRUE) { emit rx_driver.clock; wait cycle; };
  };--rx_driver_clk_tcm

  bfm_drive_init_tcm() @sig.clk_r_ev is {
    sig.full_clr_p$=0;
    sig.rx_in_p$=1;
  };--bfm_drive_init_tcm

  bfm_drive_data_tcm() @sig.clk_r_ev is {
	var rx_data_in : uart_rx_byte_s;
    wait true (sig.rst_p$ == 0);
	message(NONE, "bfm_drive_data_tcm reset wait ended ", name);
	while(TRUE) {
      rx_data_in = rx_driver.get_next_item(); 
      messagef(HIGH, 
        "bfm_drive_data_tcm Requesting new rx byte waits for %d", 
        rx_data_in.transmit_delay
      );
	  wait [rx_data_in.transmit_delay] * cycle;
	  --start bit
	  sig.rx_in_p$=0;
	  for i from 0 to 2 do {
		wait true (sig.ck_en_p$ == 1);
		wait true (sig.ck_en_p$ == 0);
	  };
	  --data bits
      var rx_bits_l : list of bit = pack(
        packing.low, rx_data_in.uart_rx_data
      );
      message(NONE, "bfm_drive_data_tcm drive byte ") {
        print rx_bits_l using hex;
      };
        
	  while(!rx_bits_l.is_empty()) {
        sig.rx_in_p$=rx_bits_l.pop();
	    for i from 0 to 2 do {
		  wait true (sig.ck_en_p$ == 1);
		  wait true (sig.ck_en_p$ == 0);
	    };--for
	  };--while
	  --stop bit
	  var stop_bit_cnt : byte;
	  gen stop_bit_cnt keeping{it >= 2 && it <= 5;};
	  sig.rx_in_p$=1;
	  for i from 0 to stop_bit_cnt do {
	    wait true (sig.ck_en_p$ == 1);
	    wait true (sig.ck_en_p$ == 0);
	  };
	  
	  emit rx_driver.item_done;
	}--while
  };--bfm_drive_data_tcm

  run() is also {
    start bfm_drive_data_tcm();
    start bfm_drive_init_tcm();
    start rx_driver_clk_tcm();
  };
};--unit uart_rx_bfm_u

'>


