
-- VHDL Test Bench Created from source file rx.vhd -- 12:34:00 11/03/2004
--
--
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;

ENTITY rx_to_tx_tb IS
END rx_to_tx_tb;

ARCHITECTURE behavior OF rx_to_tx_tb IS 

--specman
component comspec
end component;
for all: comspec use entity work.SPECMAN_REFERENCE (arch);

        CONSTANT clk_period         : time      := 20 ns;
        CONSTANT ck_period_mult     : integer   := 3;

        CONSTANT ck_en_period       : time      := clk_period * ck_period_mult;

	COMPONENT rx
	PORT(
		rx_in : IN std_logic;
		clk : IN std_logic;
		ck_en : IN std_logic;
		reset : IN std_logic;
		full_clr : IN std_logic;          
		d_out : OUT std_logic_vector(7 downto 0);
		full : OUT std_logic
		);
	END COMPONENT;

COMPONENT tx
  PORT(
    d_in : IN std_logic_vector(7 downto 0);
    load : IN std_logic;
    clk : IN std_logic;
    ck_en : IN std_logic;
    reset : IN std_logic;          
    tx_out : OUT std_logic;
    busy : OUT std_logic
  );
END COMPONENT;

	SIGNAL rx_in :  std_logic;
	SIGNAL clk :  std_logic;
	SIGNAL ck_en :  std_logic;
	SIGNAL reset :  std_logic;
	SIGNAL d_rx_to_tx :  std_logic_vector(7 downto 0);
	SIGNAL full :  std_logic;
	SIGNAL full_clr :  std_logic;

    --tx logic for load
    signal load_tx   : std_logic;
    signal load_tx_q : std_logic;--sample
    signal load_tx_p : std_logic;--one cycle pulse on rise
    signal load_tx_pq: std_logic;--one cycle pulse on rise

    signal busy   : std_logic;
    signal tx_out : std_logic;


BEGIN

uut_rx: rx PORT MAP(
	rx_in => rx_in,
	clk => clk,
	ck_en => ck_en,
	reset => reset,
	d_out => d_rx_to_tx,
	full => full,
	full_clr => full_clr
);
uut: tx PORT MAP(
  tx_out => tx_out,
  busy => busy,
  clk => clk,
  ck_en => ck_en,
  load => load_tx_pq,
  d_in => d_rx_to_tx,
  reset => reset
);



    --tx logic for load
    load_tx   <= full;
    load_tx_p <= load_tx and not(load_tx_q);
    process (clk,reset) begin
      if(reset = '1') then 
        load_tx_q <= '0';
        load_tx_pq<= '0';
      elsif(clk'event and clk = '1') then
        load_tx_q <= load_tx;
        load_tx_pq<= load_tx_p;
      end if;
    end process;

   clk_p : PROCESS
	BEGIN
	    CLOCK_LOOP : LOOP
		clk <=  '0';
		WAIT FOR clk_period/2;
		clk <= '1';
		WAIT FOR clk_period/2;
            END LOOP CLOCK_LOOP;
	END PROCESS;

   ck_en_p : PROCESS
	BEGIN
            ck_en <= '0';
            WAIT FOR clk_period/4;
	    CK_EN_LOOP : LOOP
		ck_en <=  '1';
		WAIT FOR clk_period;
		ck_en <= '0';
		WAIT FOR (ck_en_period - clk_period);
            END LOOP CK_EN_LOOP;
	END PROCESS;

   reset_p : PROCESS
	BEGIN
            reset <= '1';
            WAIT FOR 45 ns;
            reset <='0';
            WAIT;
	END PROCESS;


END;
