/*------------------------------------------------------------------------- 
File name   : uart_bfm.e
Title       : uart eVC cell structure
Project     : uart eVC
Developers  : Pinhas Krengel http://bknpk.no-ip.biz/my_web/index.html
Created     : Dec 2011
Description : This file drives data to uart TX.
Notes       : 
--------------------------------------------------------------------------- 
==== This source file may be used and distributed without ====
==== restriction provided that this copyright statement is not ====
==== removed from the file and that any derivative work contains ====
==== the original copyright notice and the associated disclaimer.====
==== ====
==== THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY ====
==== EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED ====
==== TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS ====
==== FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE AUTHOR ====
==== OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, ====
==== INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES ====
==== (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE ====
==== GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR ====
==== BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF ====
==== LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT ====
==== (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT ====
==== OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE ====
==== POSSIBILITY OF SUCH DAMAGE. ====
-------------------------------------------------------------------------*/ 
<'
package pk_tx;

unit uart_tx_bfm_u {
  tx_driver : uart_tx_driver_u;
  name      : uart_tx_env_name_t;
  sig       : sig_map_u;

  tx_driver_clk_tcm() @sig.clk_r_ev is {
    while(TRUE) { emit tx_driver.clock; wait cycle; };
  };--tx_driver_clk_tcm

  bfm_drive_init_tcm() @sig.clk_r_ev is {
    sig.load$=0;
    sig.d_in$=0;
  };--bfm_drive_init_tcm

  bfm_drive_data_tcm() @sig.clk_r_ev is {
	var tx_data_in : uart_tx_byte_s;
    wait true (sig.rst_p$ == 0);
	message(NONE, "bfm_drive_data_tcm reset wait ended ", name);
	while(TRUE) {
      tx_data_in = tx_driver.get_next_item(); 
      messagef(HIGH, 
        "bfm_drive_data_tcm Requesting new rx byte waits for %d", 
        tx_data_in.transmit_delay
      );
	  wait [tx_data_in.transmit_delay] * cycle;
      messagef(NONE, 
        "bfm_drive_data_tcm drive byte %x", 
        tx_data_in.uart_tx_data
      );
	  sig.load$=1;
	  sig.d_in$=tx_data_in.uart_tx_data;
	  wait[1];
	  sig.load$=0;
	  emit tx_driver.item_done;
	}--while
  };--bfm_drive_data_tcm

  run() is also {
    start bfm_drive_data_tcm();
    start bfm_drive_init_tcm();
    start tx_driver_clk_tcm();
  };
};--unit uart_tx_bfm_u

'>

