/*------------------------------------------------------------------------- 
File name   : uart_monitor.e
Title       : uart eVC cell structure
Project     : uart eVC
Developers  : Pinhas Krengel http://bknpk.no-ip.biz/my_web/index.html
Created     : Dec 2011
Description : The monitor unit implements a monitor for tx serail data.
Notes       : 
--------------------------------------------------------------------------- 
==== This source file may be used and distributed without ====
==== restriction provided that this copyright statement is not ====
==== removed from the file and that any derivative work contains ====
==== the original copyright notice and the associated disclaimer.====
==== ====
==== THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY ====
==== EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED ====
==== TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS ====
==== FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE AUTHOR ====
==== OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, ====
==== INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES ====
==== (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE ====
==== GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR ====
==== BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF ====
==== LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT ====
==== (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT ====
==== OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE ====
==== POSSIBILITY OF SUCH DAMAGE. ====
-------------------------------------------------------------------------*/ 
<'
package pk_tx;

unit uart_tx_monitor_u {
  name : uart_tx_env_name_t;

  has_coverage : bool;
  keep soft has_coverage == FALSE;

  scbd : list of byte; --scoreboard
  keep soft scbd.size() == 0;

  data_out_reverse_flg : bool;
  keep soft data_out_reverse_flg == FALSE;
  data_wait_cycles : uint;
  keep soft data_wait_cycles == 15;

  sig : sig_map_u;
  
  --data intergrity check
  mon_tx_data_cmp(tx_d_byte : byte) is {
    check that !scbd.is_empty() else 
	dut_error("tx_mon_u data output without input ", tx_d_byte);
	check that tx_d_byte == scbd[0] else
	dut_error(
	  "tx_mon_u data compare error dut=", tx_d_byte,
	  " exp=", scbd[0]
	);
	messagef(NONE, "%s tx_mon_u date compare ok of %x", name, tx_d_byte);
	scbd.delete(0);
  };--mon_tx_data_cmp

  mon_tx_data_tcm() @sig.clk_r_ev is {
	var txd_l     : list of bit;
    var last_bit  : bit;
    var first_bit : bit;

    wait true (sig.rst_p$ == 0);
	message(NONE, "tx_mon_u reset wait ended ", name);
	while(TRUE) {
	  --wait for data start bit
	  wait true (sig.txd$ == 0);
	  for i from 0 to 7 do {
	    wait [data_wait_cycles];
        txd_l.add(sig.txd$);
        if(i == 7 && has_coverage) then {
          last_bit=sig.txd$;--last bit for coverage
        } else {
          if(i == 0 && has_coverage) then {
            first_bit=sig.txd$;--first bit for coverage
            start cov_first_bit_after_start(first_bit);
          };
        };
	  };--for
	  --make sure last bit is stop bit
      wait [data_wait_cycles];
	  check that sig.txd$ == 1 else 
	  dut_error("tx_mon_u end bit not 1 ", sig.txd$);
      if(has_coverage) then {
        start cov_stop_bit_start_tcm();
        start cov_last_bit_before_stop(last_bit);
      };

	  --end bit was okay - check that the output data is okay too.
      if(data_out_reverse_flg) then {
        txd_l=txd_l.reverse();
      };
	  var tx_d_byte : byte;
	  unpack(packing.low, txd_l, tx_d_byte);
	  message(NONE, "tx_mon_u data\n", name) {
	    print txd_l     using hex;
		print tx_d_byte using hex;
	  };--message
	  txd_l.clear();
      mon_tx_data_cmp(tx_d_byte);
	};--while
  };--mon_tx_data_tcm

  mon_tx_data_load_tcm() @sig.clk_r_ev is {
    wait true (sig.rst_p$ == 0);
	message(NONE, "mon_tx_load_tcm reset wait ended ", name);
	while(TRUE) {
	  --wait for data start bit
	  wait true (sig.load$ == 1);
	  scbd.add(sig.d_in$);
	  messagef(HIGH, "%s mon_tx_load_tcm data in %x", name, sig.d_in$);
	  wait [1];
	};--while
  };--mon_tx_data_load_tcm

  cov_last_bit_before_stop(last_bit : bit)    @sig.clk_r_ev is {};--coverage
  cov_first_bit_after_start(first_bit : bit)  @sig.clk_r_ev is {};--coverage
  cov_stop_bit_start_tcm()                    @sig.clk_r_ev is {};--coverage
  run() is also {
    start mon_tx_data_tcm();
	start mon_tx_data_load_tcm();
  };
};--unit uart_tx_monitor_u

--coverage
extend TRUE'has_coverage uart_tx_monitor_u {
  --measures the number of cycles of a stop bit
  event cov_stop_bit_ev;
  cov_stop_bit_len : uint(bits:4);
  cov_stop_bit_start_tcm() @sig.clk_r_ev is {
    cov_stop_bit_len=0;
    while(sig.txd$ == 1) {
      wait [1] * cycle;
      if(cov_stop_bit_len <= 0xf) then {
        cov_stop_bit_len+=1;
      };
    };--while
    emit cov_stop_bit_ev;
  };
  cov_last_bit : bit;
  event cov_last_bit_ev;
  cov_last_bit_before_stop(last_bit : bit) @sig.clk_r_ev is only {
    cov_last_bit=last_bit;
    emit cov_last_bit_ev;
  };
  cov_first_bit : bit;
  event cov_first_bit_ev;
  cov_first_bit_after_start(first_bit : bit) @sig.clk_r_ev is only {
    cov_first_bit=first_bit;
    emit cov_first_bit_ev;
  };

  --coverage
  cover cov_last_bit_ev is {
    item cov_last_bit;
  };
  cover cov_first_bit_ev is {
    item cov_first_bit;
  };
  cover cov_stop_bit_ev is {
    item cov_stop_bit_len;
  };
};


'>

